;!to "bmpanimation.prg",cbm
!cpu 6510

; ______________________________________________________________________
; _______________ STATIC NUMBERS
irq_line3             = $2f
irq_line1             = $f8 ;open border part 1
irq_line2             = $fc ;$fd ;open border part 2
; ______________________________
; _________________ RAM ADRESSES
ghostbyte             = $7fff

sidinit               = $1000
sidplay               = sidinit+3

code_start            = $4000
sprite_data           = $4c00
; ______________________________________________________________________
; ______________________________________________ INCLUDES

*= $1000
  !bin "waltzblues.c64",,2

*= $2000
  !bin "bitmap2000.bin",,2
*= $6000
  !bin "bitmap6000.bin",,2
*= $a000
  !bin "bitmapA000.bin",,2
*= $e000
  !bin "bitmapE000.bin",,2

*= sprite_data
  !fill 64, $ff
  !bin "sprite-font.spr"
; ______________________________________________________________________
; ______________________________________________ CODE START
*= code_start
  jmp init

!zone irq2
irq2:
  pha
  txa
  pha
  tya
  pha

  lda #<WedgeIRQ
  sta $fffe
  lda #>WedgeIRQ
  sta $ffff
  inc $d012
  lda #$01
  sta $d019
  tsx
  cli

  !fill 8, $ea

WedgeIRQ:
  txs

  ; PAL-63  ; NTSC-64    ; NTSC-65
  ;---------;------------;-----------
  ldx #$08   ; ldx #$08   ; ldx #$09
  dex        ; dex        ; dex
  bne *-1    ; bne *-1    ; bne *-1
  bit $00    ; nop

  lda $d012
  cmp $d012  ; <- critical instruction (ZERO-Flag will indicate if Jitter = 0 or 1)
  beq *+2

  asl $d019 ; Ack IRQ

  ; Reset bit 3 for the next frame for opening border
  lda $d011
  ora #%00010000
  sta $d011

  nop
  nop
;        lda #%11101000
;        sta $d016

nts_fix_border2 = * + 1
  jsr waste28cycles
  lda lower_line_color3
  sta $d020
  sta $d021

  jsr waste53cycles
  jsr waste30cycles

  ldy charcolor
  sty $d021

  jsr waste32cycles
  ldx #$00
-
starting_point_pattern = *+1
  lda hires_pattern1, x
  sta ghostbyte
  inx
nts_fix_border3 = * +1
  jsr waste24cycles
  txa
ntsc_fix_border_end = * + 1
  cmp #40
  bne -
  nop
  nop
  nop
  lda #%00000000
  sta ghostbyte

  lda lower_line_color3
  sta $d020
  sta $d021

  jsr waste53cycles
  jsr waste53cycles
  lda bordercolor
  sta $d020
  sta $d021

  lda starting_point_pattern
ntsc_fix_border_start1 = * + 1
  sbc #$00
  sta starting_point_pattern
  cmp #<hires_pattern1
  beq +
  lda #<hires_pattern1
  clc
ntsc_fix_border_start2 = * + 1
  adc #$00
  sta starting_point_pattern
  jmp ++
+       lda #<hires_pattern2
  clc
ntsc_fix_border_start3 = * + 1
  adc #$00
  sta starting_point_pattern

++
  jsr cycle_screens
  jsr scroll_sprite_text
  ;jsr checkkey
  lda #irq_line3    ;this is how to tell at which rasterline we want the irq to be triggered
  sta $d012
  lda #<irq3        ;this is how we set up
  sta $fffe         ;the address of our interrupt code
  lda #>irq3
  sta $ffff

  pla
  tay
  pla
  tax
  pla
  rti

!zone irq1
irq1:
  sta .atemp+1
  stx .xtemp+1
  sty .ytemp+1

  lsr $d019    ;as stated earlier this might fail only on exotic HW like c65 etc.
               ;lda #$ff sta $d019 is equally fast, but uses two more bytes and
               ;trashes A

  lda $dd00
  and #%11111100
  ora #%00000010  ; VIC bank 1
  tax
  lda #%00010100
  tay
  lda #%01111100  ; screen at $1c00, bitmap at $2000

  ;jsr waste28cycles

  sta $d018
  stx $dd00
  sty $d011

  lda #irq_line2   ;this is how to tell at which rasterline we want the irq to be triggered
  sta $d012
  lda #<irq2  ;this is how we set up
  sta $fffe  ;the address of our interrupt code
  lda #>irq2
  sta $ffff

.atemp
  lda #$00
.xtemp:
  ldx #$00
.ytemp:
  ldy #$00
  rti

!zone irq3
irq3:
  sta .atemp+1
  stx .xtemp+1
  sty .ytemp+1

  lsr $d019    ;as stated earlier this might fail only on exotic HW like c65 etc.
               ;lda #$ff sta $d019 is equally fast, but uses two more bytes and
               ;trashes A

  ;jsr waste16cycles
  nop
  jsr set_current_bank
  lda #%00111000  ; bitmap mode
  sta $d011

  lda SYSTEM_TYPE
  cmp #$03
  beq +
  inc ntsc_frameskip
  lda ntsc_frameskip
  cmp #$07
  bne +
  lda #$00
  sta ntsc_frameskip
  jmp ++
+       lda #$00
  ldx #$00
  ldy #$00
  jsr sidplay
++
  lda #irq_line1   ;this is how to tell at which rasterline we want the irq to be triggered
  sta $d012
  lda #<irq1  ;this is how we set up
  sta $fffe  ;the address of our interrupt code
  lda #>irq1
  sta $ffff

.atemp
  lda #$00
.xtemp:
  ldx #$00
.ytemp:
  ldy #$00
  rti

irq_init:
  lda #$7f
  sta $dc0d  ;disable timer interrupts which can be generated by the two CIA chips
  sta $dd0d  ;the kernal uses such an interrupt to flash the cursor and scan the keyboard, so we better
             ;stop it.

  lda $dc0d  ;by reading this two registers we negate any pending CIA irqs.
  lda $dd0d  ;if we don't do this, a pending CIA irq might occur after we finish setting up our irq.
             ;we don't want that to happen.

  lda #$01   ;this is how to tell the VICII to generate a raster interrupt
  sta $d01a

  lda #irq_line3   ;this is how to tell at which rasterline we want the irq to be triggered
  sta $d012

  lda #%00011000   ;as there are more than 256 rasterlines, the topmost bit of $d011 serves as
  sta $d011  ;the 8th bit for the rasterline we want our irq to be triggered.
             ;here we simply set up a character screen, leaving the topmost bit 0.

  lda #$35   ;we turn off the BASIC and KERNAL rom here
  sta $01    ;the cpu now sees RAM everywhere except at $d000-$e000, where still the registers of
             ;SID/VICII/etc are visible

  lda #<irq3  ;this is how we set up
  sta $fffe  ;the address of our interrupt code
  lda #>irq3
  sta $ffff

  rts

SYSTEM_TYPE:
  !byte $03

ntsc_frameskip:
  !byte $00
bordercolor:
  !byte $00
charcolor:
  !byte $06
lower_line_color3:
  !byte $0c
; ______________________________________________________________________
init_vic_all:
  lda #$00
  sta $d020 ; Border Color
  ;lda #$00 ;BACKGROUND
  sta $d021 ; Screen Color

  lda #%00111011  ; bitmap mode
  sta $d011
  lda #%11101000  ; single-colour mode
  sta $d016
  rts

set_bank0:
  lda #%00011000  ; screen at $0400, bitmap at $2000
  sta $d018

  lda $dd00
  and #%11111100
  ora #%00000011  ; VIC bank 0
  sta $dd00
  jmp set_current_bank_return

set_bank1:
  lda #%01111100  ; screen at $1c00, bitmap at $2000
  sta $d018

  lda $dd00
  and #%11111100
  ora #%00000010  ; VIC bank 1
  sta $dd00
  jmp set_current_bank_return

set_bank2:
  lda #%00111100  ; screen at $0c00, bitmap at $2000
  sta $d018

  lda $dd00
  and #%11111100
  ora #%00000001  ; VIC bank 2
  sta $dd00
  jmp set_current_bank_return

set_bank3:
  lda #%00111100  ; screen at $0c00, bitmap at $2000
  sta $d018

  lda $dd00
  and #%11111100
  ora #%00000000  ; VIC bank 3
  sta $dd00
  jmp set_current_bank_return

current_phase:
  !byte $04
current_speedmod_phase:
  !byte $01
speedmod:
  !byte $03

set_current_bank:
  jmp set_bank0 ;selfmod
set_current_bank_return:
  rts

cycle_screens:
  dec current_speedmod_phase
  bne cycle_screens_end
  lda speedmod
  sta current_speedmod_phase

  lda current_phase
  cmp #$01
  bne +
  lda #<set_bank0
  sta set_current_bank+1
  lda #>set_bank0
  sta set_current_bank+2
  jmp cycle_screens_return
+
  cmp #$02
  bne +
  lda #<set_bank1
  sta set_current_bank+1
  lda #>set_bank1
  sta set_current_bank+2
  jmp cycle_screens_return
+
  cmp #$03
  bne +
  lda #<set_bank2
  sta set_current_bank+1
  lda #>set_bank2
  sta set_current_bank+2
  jmp cycle_screens_return
+
  lda #<set_bank3
  sta set_current_bank+1
  lda #>set_bank3
  sta set_current_bank+2

;set_current_bank:
;        lda current_phase
;        cmp #$01
;        bne +
;        jmp set_bank0
;+       cmp #$02
;        bne +
;        jmp set_bank1
;+       cmp #$03
;        bne +
;        jmp set_bank2
;+       jmp set_bank3
;set_current_bank_return:
;        rts
;
;cycle_screens:
;        dec current_speedmod_phase
;        bne cycle_screens_end
;        lda speedmod
;        sta current_speedmod_phase
;        ;jsr set_current_bank

cycle_screens_return:
  dec current_phase
  bne cycle_screens_end
  lda #$04
  sta current_phase
cycle_screens_end:
  rts
; ______________________________________________________________________
VIC_BANK             = $4000
SCREEN_MEM1          = VIC_BANK + $1c00
SPRITE_DATA_0_MEM1  = SCREEN_MEM1 + $03f8

sprites_start       = ((sprite_data-VIC_BANK)/64)

VIC_SPRITE_SWITCH   = $d015 ;Each bit in $d015 controls whether the corresponding sprite is displayed or not. The lowest bit controls Sprite0 and so on.
VIC_SPRITE_MODE     = $d01c ;Each bit in $d01c controls whether the corresponding sprite is displayed in multicolor mode or not. The lowest bit controls Sprite0 and so on.
VIC_SPRITE_PRIORITY = $d01b
VIV_SPRITE_STETCH_V = $d017
VIV_SPRITE_STETCH_H = $d01d

;The X coordinates are 9 bit long, and the 9th bit of all sprites are collected at $d010, similarly as $d015 works.
SPRITE_H_POSITION_0 = $d000
SPRITE_V_POSITION_0 = $d001
SPRITE_H_POSITION_1 = $d002
SPRITE_V_POSITION_1 = $d003
SPRITE_H_POSITION_2 = $d004
SPRITE_V_POSITION_2 = $d005
SPRITE_H_POSITION_3 = $d006
SPRITE_V_POSITION_3 = $d007
SPRITE_H_POSITION_4 = $d008
SPRITE_V_POSITION_4 = $d009
SPRITE_H_POSITION_5 = $d00A
SPRITE_V_POSITION_5 = $d00B
SPRITE_H_POSITION_6 = $d00C
SPRITE_V_POSITION_6 = $d00D
SPRITE_H_POSITION_7 = $d00E
SPRITE_V_POSITION_7 = $d00F
SPRITE_H_MSB        = $d010

SPRITE_COLOR_0      = $d027
SPRITE_COLOR_1      = $d028
SPRITE_COLOR_2      = $d029
SPRITE_COLOR_3      = $d02A
SPRITE_COLOR_4      = $d02B
SPRITE_COLOR_5      = $d02C
SPRITE_COLOR_6      = $d02D
SPRITE_COLOR_7      = $d02E

init_sprites:
  jsr empty_sprites
;          lda #((sprite_data-VIC_BANK)/64)+1
;          jsr set_all_sprites_to_a
  jsr init_x_position

  lda lower_line_color3
  jsr set_spritecolors_to_a

  lda #%00000000
  sta VIC_SPRITE_MODE
  sta VIC_SPRITE_PRIORITY

  lda #%11111111
  ;sta VIC_SPRITE_SWITCH
  sta VIV_SPRITE_STETCH_V
  sta VIV_SPRITE_STETCH_H
;          jsr enable_all_sprites
;          rts

enable_all_sprites:
  lda #%11111111
  sta VIC_SPRITE_SWITCH
  rts

disable_all_sprites:
  lda #%00000000
  sta VIC_SPRITE_SWITCH
  rts

set_spritecolors_to_a:
  ldx #$08
-
  sta SPRITE_COLOR_0-1,x
  dex
  bne -
  rts

empty_sprites:
  lda #sprites_start
set_all_sprites_to_a:
  ldx #$08
-
  sta SPRITE_DATA_0_MEM1-1,x
  dex
  bne -
  rts

init_x_position:
  ldx #16
-         lda sprite_init_positions-1,x
  sta SPRITE_H_POSITION_0-1,x
  dex
  bne -
  lda #%11100000
  sta SPRITE_H_MSB
  lda #$00
  sta current_offset
  rts

current_offset:
  !byte $00
move_sprites:
  ldx #$08
  ldy #$00
-
  lda SPRITE_H_POSITION_0,y
  clc
  adc #$fc
  sta SPRITE_H_POSITION_0,y
  iny
  iny
  dex
  bne -

  inc current_offset

.compare:
  lda lower_line_color3
  jsr set_spritecolors_to_a

  lda current_offset
  cmp #$02
  bne +

  lda #%11000000
  sta SPRITE_H_MSB

+         lda current_offset
  cmp #$06
  bne ++

  lda #%11000001
  sta SPRITE_H_MSB
  lda SPRITE_H_POSITION_0
;  lda #247
  clc
ntsc_sprite_position_patch = * +1
  adc #$f8
  sta SPRITE_H_POSITION_0

++        lda current_offset
  cmp #12
  bne +++
  inc scrollflag
  jsr init_x_position
+++       rts

scroll_sprite_text:
  jsr move_sprites
  lda scrollflag
  cmp #$01
  bne +
  dec scrollflag
  jsr reposition_sprite_pointers
+         rts

reposition_sprite_pointers
  ldx #$00
-         lda SPRITE_DATA_0_MEM1+1,x
  sta SPRITE_DATA_0_MEM1,x
  inx
  cpx #$07
  bne -
.pointer_hi = *+2
.pointer_lo = *+1
.re_enter:
  lda sprite_text
  tax
  clc
  adc #((sprite_data-VIC_BANK)/64)
  sta SPRITE_DATA_0_MEM1+7
  cpx #$ff
  bne +
  lda #<sprite_text
  sta .pointer_lo
  lda #>sprite_text
  sta .pointer_hi
  jmp .re_enter:
+
  inc .pointer_lo
  bne ++
  inc .pointer_hi
++
  rts

scrollflag:
  !byte $00

sprite_init_positions:
  !byte $17,$ff,$47,$ff,$77,$ff,$A7,$ff
  !byte $D7,$ff,$07,$ff,$37,$ff,$67,$ff
; ______________________________________________________________________
sprite_text:
  !convtab "convtab.ct"
  !text "           thx for watching the mayday show! love goes out to: abyss connection, alpha flight 1970, arise, arkanix labs, arsenic, artline designs, atlantis, avatar, backbone society, beyond force, booze design, c64 club berlin fka bcc, camelot, cascade, censor design, chorus, c.i.a., code7, cosine, covert bitops, creators, crescent, crest, dacapo, darklite, delysid, delta machine, desire, dinasours, dmagic, dream, dreams, elysium, exceed, excess, extend, f4cg, fairlight, flood, genesis project, hack n'trade, hitmen, hoaxers, hokuto force, hvsc crew, insane, krynn ltd, kryo, laxity, luneart, multistyle labs, noice, no name, nostalgia, nuance, offence, onslaught, oxyron, padua, panda design, plush, prosonix, rabenauge, razor 1911, react, resource, return, rgcd, role, samar, saufbox, sauf park, sentenced to death, shape, singular, sitzgruppe, stonehead, style, svenonacid, sys5, the new dimension, triad, tropyx, trsi, vision, vulture design, wow, wrath designs."
  !byte $ff ; end-sign
; ______________________________________________________________________
GETIN               = $ffe4
RESET               = $FCE2
!zone check_key
.keydown:
	!byte $00       ;byte of keydown

checkkey:
 lda SYSTEM_TYPE
 cmp #$03
 bne .no_key
	lda $dc01
	cmp .keydown
	bne .newkey
	lda #$ff      ;returns $ff in no change
	jmp .no_key
.newkey:
	sta .keydown
        cmp #$fe      ;new key is 1
	beq .key_1
        lda .keydown
	cmp #$ef      ;new key is spacebar
	beq .key_space
.no_key:
        rts
.key_space:

reset_in_irq:

	lda #$00
	tax
	tay
	jsr sidinit
        jsr disable_all_sprites
	lda #$01
        sta $d021
        sta $d020
;        jsr fill_color_with_a

	asl $d019   ;Ack all IRQ's
	lda $dc0d
	lda $dd0d
	lda #$81    ;reset CIA 1 IRQ
	ldx #$00    ;remove raster IRQ
	ldy #$37    ;reset MMU to roms
	sta $dc0d
	stx $d01a
	sty $01
	ldx #$ff    ;clear the stack
	txs
	cli         ;reenable IRQ's
	jmp RESET

.key_1:
  lda VIV_SPRITE_STETCH_V
  cmp #$00
ntsc_size_scroller:
  beq .other
  lda #$00
  sta VIV_SPRITE_STETCH_V
  lda #19
  sta ntsc_fix_border_end
  lda #$01
  sta ntsc_fix_border_start1
  sta ntsc_fix_border_start2
  sta ntsc_fix_border_start3
  rts
.other:
  lda #$ff
  sta VIV_SPRITE_STETCH_V
  lda #40
  sta ntsc_fix_border_end
  lda #$00
  sta ntsc_fix_border_start1
  sta ntsc_fix_border_start2
  sta ntsc_fix_border_start3
.exit:
  rts
; ______________________________________________________________________
init:
        sei ; maskierbare IRQs ausschalten
        jsr init_sprites
        jsr ntsc_size_scroller

        lda #$08
        sta $d016

        lda #$00
        ;tax
        ;tay
        jsr sidinit

        ;init colormem
        lda #$10
        ldx #$00
.COLORLOOP
        sta $0400,x
        sta $0500,x
        sta $0600,x
        sta $06e8,x

        sta $5c00,x
        sta $5d00,x
        sta $5e00,x
        sta $5ee8,x

        sta $8c00,x
        sta $8d00,x
        sta $8e00,x
        sta $8ee8,x

        sta $cc00,x
        sta $cd00,x
        sta $ce00,x
        sta $cee8,x

        inx
        bne .COLORLOOP
        lda #$00
        sta $cfff ; hide the irq address points with black/black

        jsr irq_init

        cli
loop:
        jmp loop
;*=$1000
;sidinit:
;sidplay = *+3
;  !binary "bin/waltzblues.sid",, 0x7E

;*=$2000 ;Bitmap
;  !binary "bin/dvdp2_1_bitmap.PRG",8000,2

hires_pattern1:
!binary "pattern1-fontdata.bin",,
!byte $ff
hires_pattern2:
!binary "pattern2-fontdata.bin",,
!byte $ff

waste53cycles:
        bit $ea ; 3
        nop     ; 2
waste48cycles:
        nop     ; 2
waste46cycles:
        nop     ; 2
waste44cycles:
        nop     ; 2
waste42cycles:
        nop     ; 2
waste40cycles:
        nop     ; 2
waste38cycles:
        nop     ; 2
waste36cycles:
        nop     ; 2
waste34cycles:
        nop     ; 2
waste32cycles:
        nop     ; 2
waste30cycles:
        nop     ; 2
waste28cycles:
        nop     ; 2
waste26cycles:
        nop     ; 2
waste24cycles:
        nop     ; 2
waste22cycles:
        nop     ; 2
waste20cycles:
        nop     ; 2
waste18cycles:
        nop     ; 2
waste16cycles:
        nop     ; 2
waste14cycles:
        nop     ; 2
waste12cycles:
        bit $ea ; 3
waste9cycles:
        rts     ; 6 + 3 for the jsr

