!cpu 6510

!src "colorcodes.asm"

!if RELEASE = 1 {
  !src "../framework/frameworksymbols.inc"
}
; ______________________________________________________________________
; _______________ STATIC NUMBERS

enable                = $20 ; OPCODE 'JSR'
disable               = $2c ; OPCODE 'BIT'

; ______________________________
; ____________ ZEROPAGE ADRESSES

savea                 = $02
savex                 = $03
savey                 = $04
; ______________________________
;         $05 - $DF free to use!

addvalue              = $05
addspeed              = addvalue+1

zp_temp               = addspeed+1
zp_temp_lo            = zp_temp
zp_temp_hi            = zp_temp_lo+1

part_ct               = zp_temp_hi+1
part_ct_lo            = part_ct
part_ct_hi            = part_ct_lo+1

virtual_x             = part_ct_hi+1
virtual_x0            = virtual_x
virtual_x1            = virtual_x+1
virtual_x2            = virtual_x+2
virtual_x3            = virtual_x+3
virtual_x4            = virtual_x+4
virtual_x5            = virtual_x+5
virtual_x6            = virtual_x+6
virtual_x7            = virtual_x+7

virtual_y             = virtual_x+8
virtual_y0            = virtual_y
virtual_y1            = virtual_y+1
virtual_y2            = virtual_y+2
virtual_y3            = virtual_y+3
virtual_y4            = virtual_y+4
virtual_y5            = virtual_y+5
virtual_y6            = virtual_y+6
virtual_y7            = virtual_y+7
; ______________________________
; $E0 - (max.) $F$  KRILL LOADER
; ______________________________
color                 = $f5

irq_ready             = $f6
d012_value            = $f7
d011_value            = $f8
irq_lo                = $f9
irq_hi                = $fa

global_ct             = $fb             ; $fb, $fc, $fd
; ______________________________
; _________________ RAM ADRESSES

vicbank               = $4000
vidmem0               = vicbank+$2000
vidmem1               = vicbank+$2400
charset               = vicbank+$2800
sprite_data           = vicbank+$3000
sprite_base           = <((sprite_data-vicbank)/$40)

plasma_kot_vidmem0	  = $8000           ; speedcode 1
plasma_kot_vidmem1	  = $a329           ; speedcode 2

char_xor_val          = (<(((vidmem0-vicbank)/$400) << 4)+ <(((charset-vicbank)/$800) << 1))XOR(<(((vidmem1-vicbank)/$400) << 4)+ <(((charset-vicbank)/$800) << 1))
scr_off_xor_val       = $1b XOR $6b
; ______________________________________________________________________
; ______________________________________________ INCLUDES
!if RELEASE != 1 {
*= $0801
  !bin "../framework/framework-noloader.c64",,2
}

*= $2ffe
  !byte <plasma_code_start
  !byte >plasma_code_start

*= charset
  !bin "dither255.chr"

*= sprite_data
  !bin "sprites.bin"
; ______________________________________________________________________
; ______________________________________________ CODE START
*= $3000
plasma_code_start:
  jsr init_vic_plasma

  jsr gen_plasma_kot0

  lda #$01
  sta addvalue

;  lda $1035
;-
;  cmp $1035
;  beq -

  lda #$00
  sta part_ct_lo
  sta part_ct_hi

  lda scr_off_switch
  sta d011_value

  lda #black
  sta color

  ; #$30
  lda #$93
  sta d012_value

  lda #<irq0
  sta irq_lo
  lda #>irq0
  sta irq_hi

  lda #$04
  sta addspeed
; ______________________________________________ FADE IN
fade_in:

  jsr wait_irq
  dec fade_in_line0
  inc fade_in_line1
  lda fade_in_line0
  cmp #$30
  bne fade_in

  lda #$fa
  sta fade_in_line1

  jsr wait_irq

  ldx #14
-
  lda raster_fade_tab+3,x
  sta color
  tay
  jsr wait_irq
  sty border_col_top
  jsr wait_irq
  jsr wait_irq
  dex
  bpl -

  lda #light_blue
  sta stop_hit

  lda $1035
-
  cmp $1035
  beq -

  lda #0
  sta part_ct_hi
  sta part_ct_lo

  lda # <(((vidmem1-vicbank)/$400) << 4)+ <(((charset-vicbank)/$800) << 1)
  sta $d018                 ; set screen/charset in $d018

  lda scr_off_switch
  eor #scr_off_xor_val
  sta scr_off_switch

  lda #enable
  sta enable_sprites

  lda #%11111111
  sta $d015
; ______________________________________________ MAINLOOP
mainloop_plasma:

  jsr plasma_loop

  jsr gen_plasma_kot1
  jsr plasma_loop

  jsr gen_plasma_kot2
  jsr plasma_loop

  jsr gen_plasma_kot3
  jsr plasma_loop

  jsr gen_plasma_kot4
  lda $d018
  eor #char_xor_val
  sta $d018
  jsr plasma_loop

-
wait_end =*+1
  lda #$00
  beq -

  jsr wait_irq

  lda #blue
  sta stop_hit

  lda scr_off_switch
  eor #scr_off_xor_val
  sta scr_off_switch

  jsr wait_irq

  ldx #14
-
  lda raster_fade_tab,x
  sta color
  tay
  jsr wait_irq
  sty border_col_top
  jsr wait_irq
  jsr wait_irq
  dex
  bpl -

  jsr wait_irq

  ldx #16
-
  lda raster_fade_tab,x
  sta stop_hit
  sta stop_snare
  sta snare_hit_col
  jsr wait_irq
  jsr wait_irq
  dex
  bpl -

  jsr wait_irq

  lda #black
  sta color

  lda #$00
  sta d011_value
  sta d012_value
  sta $d012

  lda #$00
  sta irq_lo
  sta $fffe
  lda #$0f
  sta irq_hi
  sta $ffff
!if RELEASE = 1 {
  jmp back2framework
} else {
  jmp *
}
; ______________________________________________ WAIT IRQ
wait_irq:
  lda #$00
  sta irq_ready
-
  lda irq_ready
  beq -
  rts
; ______________________________________________ PLASMA LOOP
; this processes two frames of plasma speed code
; and then waits for next frame before looping again
; even with speedcode time is not enough to calc 1 frame per frame
; this is not counted, it's more "guessing" and hoping
; I get no framedrops or tearing this way ;-)
!zone PLASMA_LOOP
plasma_loop:
plasma_loop_inner:
  jsr plasma_kot_vidmem0

  clc
  lda addvalue
  adc addspeed
  sta addvalue

  jsr plasma_kot_vidmem1

  clc
  lda addvalue
  adc addspeed
  sta addvalue

.mod01 =*+1
  ldx #0
  lda addspeed_table,x
  sta addspeed
  inc .mod01

  jsr wait_irq

exit_plasma_loop =*+1
  lda #0
  bne +

  jmp plasma_loop_inner
+
  lda #0
  sta exit_plasma_loop

  lda # <(((vidmem0-vicbank)/$400) << 4)+ <(((charset-vicbank)/$800) << 1)
  sta $d018                 ; set screen/charset in $d018
  rts
; ______________________________________________________________________
!zone INIT_VIC_AND_SCREENS
; ______________________________________________ INIT VIC
init_vic_plasma:
  ldx #0
  lda #pink
-
  sta $d800+$000,x
  sta $d800+$100,x
  sta $d800+$200,x
  sta $d800+$300,x
  inx
  bne -

  lda #<!(vicbank/$4000) & 3
  sta $dd00

  lda # <(((vidmem0-vicbank)/$400) << 4)+ <(((charset-vicbank)/$800) << 1)
  sta $d018                 ; set screen/charset in $d018

init_sprites:
  ldx #$03
-
  lda #light_blue
  sta $d027,x
  lda #$7b
  sta virtual_y,x
  lda #blue
  sta $d02b,x
  lda #$7c
  sta virtual_y+4,x
  lda #$a8
  sta virtual_x,x
  dex
  bpl -

  lda #$ff
  sta $d017
  sta $d01d

  lda #sprite_base+3
  sta vidmem0+$3f8
  sta vidmem1+$3f8
  sta vidmem0+$3f8+4
  sta vidmem1+$3f8+4

  lda #sprite_base
  sta vidmem0+$3f9
  sta vidmem1+$3f9
  sta vidmem0+$3f9+4
  sta vidmem1+$3f9+4

  lda #sprite_base+1
  sta vidmem0+$3fa
  sta vidmem1+$3fa
  sta vidmem0+$3fa+4
  sta vidmem1+$3fa+4

  lda #sprite_base+2
  sta vidmem0+$3fb
  sta vidmem1+$3fb
  sta vidmem0+$3fb+4
  sta vidmem1+$3fb+4

  jsr sprites_update
  rts
; ______________________________________________________________________
!zone IRQs
!align 255,0
irq0:
  sta savea
  stx savex
  sty savey

  ldx #$08
-
  dex
  bne -

  lda #black
  sta $d020
  sta $d021

  ldx #$16
-
  dex
  bne -
scr_off_switch =*+1
  lda #$6b
  sta $d011

snare_hit_col =*+1
  lda #blue
  sta $d020
  lda #red
  sta $d021

enable_sprites:
  bit sprites_wait

fade_in_line1 =*+1
  ; #$fa
  lda #$98
  sta $d012

  lda #<irq1
  sta $fffe
  lda #>irq1
  sta $ffff

  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti
; ______________________________________________________________________
;!align 255,0
irq1:
  sta savea
  stx savex
  sty savey

  ldx #$08
-
  dex
  bne -

  lda #black
  sta $d020
  sta $d021

  ldx #$16
-
  dex
  bne -

  nop
  nop
border_col_top =*+1
  lda #black
  sta $d020
  sta $d021

  lda $d018
  eor #char_xor_val
  sta $d018

  lda part_ct_hi
  cmp #$02
  bne +

  lda #0
  sta part_ct_hi

  lda #1
  sta exit_plasma_loop

+
  clc
  lda part_ct
  adc #$01
  sta part_ct
  lda part_ct+1
  adc #$00
  sta part_ct+1

  lda $104c
  cmp #$03
  bne +
stop_hit =*+1
  ;lda #light_blue
  lda #blue
  !byte $2c
+
stop_snare =*+1
  lda #blue
  sta snare_hit_col

  lda #$00
  sta $d012

fade_in_line0 =*+1
  lda #$93
  sta d012_value

  lda scr_off_switch
  sta $d011
  sta d011_value

  lda #$00
  sta $fffe
  lda #$0f
  sta $ffff

  asl $d019
  lda savea
  ldx savex
  ldy savey
  rti
; ______________________________________________________________________
!zone GENERATE_PLASMA_KOT
; ______________________________________________ GEN KOT 0
gen_plasma_kot0:
  lda #<plasma_kot_vidmem0
  sta zp_temp_lo
  lda #>plasma_kot_vidmem0
  sta zp_temp_hi

  lda #<plasma_tab0
  sta .c01
  lda #>plasma_tab0
  sta .c02

  lda #<vidmem0
  sta .c03
  lda #>vidmem0
  sta .c04

  jsr generatorez

  lda #<plasma_kot_vidmem1
  sta zp_temp_lo
  lda #>plasma_kot_vidmem1
  sta zp_temp_hi

  lda #<plasma_tab0
  sta .c01
  lda #>plasma_tab0
  sta .c02

  lda #<vidmem1
  sta .c03
  lda #>vidmem1
  sta .c04

  jsr generatorez

  rts
; ______________________________________________ GEN KOT 1
gen_plasma_kot1:
  lda #<plasma_kot_vidmem0
  sta zp_temp_lo
  lda #>plasma_kot_vidmem0
  sta zp_temp_hi

  lda #<plasma_tab1
  sta .c01
  lda #>plasma_tab1
  sta .c02

  lda #<vidmem0
  sta .c03
  lda #>vidmem0
  sta .c04

  jsr generatorez

  lda #<plasma_kot_vidmem1
  sta zp_temp_lo
  lda #>plasma_kot_vidmem1
  sta zp_temp_hi

  lda #<plasma_tab1
  sta .c01
  lda #>plasma_tab1
  sta .c02

  lda #<vidmem1
  sta .c03
  lda #>vidmem1
  sta .c04

  jsr generatorez

  rts
; ______________________________________________ GEN KOT 2
gen_plasma_kot2:
  lda #<plasma_kot_vidmem0
  sta zp_temp_lo
  lda #>plasma_kot_vidmem0
  sta zp_temp_hi

  lda #<plasma_tab2
  sta .c01
  lda #>plasma_tab2
  sta .c02

  lda #<vidmem0
  sta .c03
  lda #>vidmem0
  sta .c04

  jsr generatorez

  lda #<plasma_kot_vidmem1
  sta zp_temp_lo
  lda #>plasma_kot_vidmem1
  sta zp_temp_hi

  lda #<plasma_tab2
  sta .c01
  lda #>plasma_tab2
  sta .c02

  lda #<vidmem1
  sta .c03
  lda #>vidmem1
  sta .c04

  jsr generatorez

  rts
; ______________________________________________ GEN KOT 3
gen_plasma_kot3:
  lda #<plasma_kot_vidmem0
  sta zp_temp_lo
  lda #>plasma_kot_vidmem0
  sta zp_temp_hi

  lda #<plasma_tab3
  sta .c01
  lda #>plasma_tab3
  sta .c02

  lda #<vidmem0
  sta .c03
  lda #>vidmem0
  sta .c04

  jsr generatorez

  lda #<plasma_kot_vidmem1
  sta zp_temp_lo
  lda #>plasma_kot_vidmem1
  sta zp_temp_hi

  lda #<plasma_tab3
  sta .c01
  lda #>plasma_tab3
  sta .c02

  lda #<vidmem1
  sta .c03
  lda #>vidmem1
  sta .c04

  jsr generatorez

  rts
; ______________________________________________ GEN KOT 4
gen_plasma_kot4:
  lda #<plasma_kot_vidmem0
  sta zp_temp_lo
  lda #>plasma_kot_vidmem0
  sta zp_temp_hi

  lda #<plasma_tab4
  sta .c01
  lda #>plasma_tab4
  sta .c02

  lda #<vidmem0
  sta .c03
  lda #>vidmem0
  sta .c04

  jsr generatorez

  lda #<plasma_kot_vidmem1
  sta zp_temp_lo
  lda #>plasma_kot_vidmem1
  sta zp_temp_hi

  lda #<plasma_tab4
  sta .c01
  lda #>plasma_tab4
  sta .c02

  lda #<vidmem1
  sta .c03
  lda #>vidmem1
  sta .c04

  jsr generatorez

  rts
; ______________________________________________ GENERATOREZ!
; this is the code generator
; it generates code like:
; !for j, 0, 24 {
;   !for i, 0, 39 {
;     clc
;     lda plasma_tabX+(j*40)+i
;     adc addval
;     sta vidmemX+(j*40)+i
;   }
; }
generatorez:
  lda #24
  sta .rowcounter

.rows
  ldx #39
.columns
  ldy #0

  lda #$18                          ; opcode CLC
  sta (zp_temp),y

  iny
  lda #$ad                          ; opcode LDA $xxxx
  sta (zp_temp),y

  iny
.c01 =*+1
  lda #<plasma_tab0
  sta (zp_temp),y

  iny
.c02 =*+1
  lda #>plasma_tab0
  sta (zp_temp),y

  iny

  lda #$65                          ; opcode ADC $xx
  sta (zp_temp),y

  iny
  lda #addvalue
  sta (zp_temp),y

  iny
  lda #$8d                          ; opcode STA $xxxx
  sta (zp_temp),y

  iny
.c03 =*+1
  lda #<vidmem0
  sta (zp_temp),y

  iny
.c04 =*+1
  lda #>vidmem0
  sta (zp_temp),y

  clc
  lda .c01
  adc #$01
  sta .c01
  bcc +
  inc .c02
+
  clc
  lda .c03
  adc #$01
  sta .c03
  bcc +
  inc .c04
+
  clc
  lda zp_temp_lo
  adc #$09
  sta zp_temp_lo
  bcc +
  inc zp_temp_hi
+
  dex
  bpl .columns

  dec .rowcounter
.rowcounter =*+1
  lda #0
  bmi +

  jmp .rows

+
  sec
  lda zp_temp_lo
  sbc #$09
  sta zp_temp_lo
  bcs +
  dec zp_temp_hi
+
  iny
  lda #$60                        ; opcode RTS
  sta (zp_temp),y

  rts
; ______________________________________________________________________
!zone SPRITES
sprites_update:
  lda #$00
  sta d010_nibble

  ldx #$03
  ldy #$06
-
  lda virtual_x,x
  asl
  sta $d000,y
  rol d010_nibble
  lda virtual_y,x
  sta $d001,y
  lda $d000,y
  adc #1
  sta $d008,y
  clc
  lda $d001,y
  adc #1
  sta $d009,y
  dey
  dey
  dex
  bpl -

d010_nibble =*+1
  lda #0
  asl
  asl
  asl
  asl
  sta $d010

  lda $d010
  ora d010_nibble
  sta $d010

  rts
; ______________________________________________________________________
sprites_wait:
  lda .curpatt
  bne +

  lda $1035
  sta .curpatt
+
  lda $1035
.curpatt =*+1
  cmp #$00
  bne +
  rts
+
  lda #$00
  sta .curpatt
next_lo =*+1
  lda #<sprites_00
  sta enable_sprites+1
next_hi =*+1
  lda #>sprites_00
  sta enable_sprites+2
  rts

sprites_00:

.savex_00 =*+1
  ldx #0

  lda virtual_x0
  cmp #$2d
  beq ++
  dec virtual_x0
  lda sprite_anim_tab0,x
  sta vidmem0+$3f8
  sta vidmem1+$3f8
  sta vidmem0+$3f8+4
  sta vidmem1+$3f8+4
  inx
  cpx #$20
  bne +
  ldx #0
+
  stx .savex_00
  jsr sprites_update

  rts
++
  lda sprite_anim_tab0,x
  sta vidmem0+$3f8
  sta vidmem1+$3f8
  sta vidmem0+$3f8+4
  sta vidmem1+$3f8+4

  inx
  cpx #$20
  beq +
  stx .savex_00

  rts
+
  lda #<sprites_01
  sta enable_sprites+1
  lda #>sprites_01
  sta enable_sprites+2

  rts

sprites_01:

.savex_01 =*+1
  ldx #0

  lda virtual_x1
  cmp #$2d+24
  beq ++
  dec virtual_x1
  lda sprite_anim_tab1,x
  sta vidmem0+$3f9
  sta vidmem1+$3f9
  sta vidmem0+$3f9+4
  sta vidmem1+$3f9+4
  inx
  cpx #$20
  bne +
  ldx #0
+
  stx .savex_01
  jsr sprites_update

  rts
++
  lda sprite_anim_tab1,x
  sta vidmem0+$3f9
  sta vidmem1+$3f9
  sta vidmem0+$3f9+4
  sta vidmem1+$3f9+4

  inx
  cpx #$20
  beq +
  stx .savex_01

  rts
+

  lda #<sprites_02
  sta enable_sprites+1
  lda #>sprites_02
  sta enable_sprites+2

  rts

sprites_02:

.savex_02 =*+1
  ldx #0

  lda virtual_x2
  cmp #$2d+(2*24)
  beq ++
  dec virtual_x2
  lda sprite_anim_tab2,x
  sta vidmem0+$3fa
  sta vidmem1+$3fa
  sta vidmem0+$3fa+4
  sta vidmem1+$3fa+4
  inx
  cpx #$20
  bne +
  ldx #0
+
  stx .savex_02
  jsr sprites_update

  rts
++
  lda sprite_anim_tab2,x
  sta vidmem0+$3fa
  sta vidmem1+$3fa
  sta vidmem0+$3fa+4
  sta vidmem1+$3fa+4

  inx
  cpx #$20
  beq +
  stx .savex_02

  rts
+

  lda #<sprites_03
  sta enable_sprites+1
  lda #>sprites_03
  sta enable_sprites+2

  rts

sprites_03:

.savex_03 =*+1
  ldx #0

  lda virtual_x3
  cmp #$2d+(3*24)
  beq ++
  dec virtual_x3
  lda sprite_anim_tab3,x
  sta vidmem0+$3fb
  sta vidmem1+$3fb
  sta vidmem0+$3fb+4
  sta vidmem1+$3fb+4
  inx
  cpx #$10
  bne +
  ldx #0
+
  stx .savex_03
  jsr sprites_update

  rts
++
  lda sprite_anim_tab3,x
  sta vidmem0+$3fb
  sta vidmem1+$3fb
  sta vidmem0+$3fb+4
  sta vidmem1+$3fb+4

  inx
  cpx #$10
  beq +
  stx .savex_03

  rts
+
  lda #<sprites_04
  sta next_lo
  lda #>sprites_04
  sta next_hi

  lda #<sprites_wait
  sta enable_sprites+1
  lda #>sprites_wait
  sta enable_sprites+2

  clc
  lda $1035
  adc #$03
  sta .sp04_endcheck

  rts

sprites_04:

.savex_04 =*+1
  ldx #$84

  lda sprites_y_sin,x
  sta virtual_y0
  sta virtual_y2

.savex_06 =*+1
  ldx #$84

  lda sprites_y_sin,x
  sta virtual_y1
  sta virtual_y3

.savex_05 =*+1
  ldx #$96

  lda sprites_x_sin,x
  sta virtual_x0

.savex_07 =*+1
  ldx #$78

  lda sprites_x_sin,x
  sta virtual_x2

.savex_08 =*+1
  ldx #$87

  lda sprites_x_sin,x
  sta virtual_x1

.savex_09 =*+1
  ldx #$68

  lda sprites_x_sin,x
  sta virtual_x3

  jsr sprites_update

  inc .savex_04
  dec .savex_06
  inc .savex_05
  dec .savex_07
  inc .savex_08
  dec .savex_09

  lda snare_hit_col
  cmp #light_blue
  bne +
  lda #white
  !byte $2c
+
  lda #light_blue
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  eor #$08
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e

.sp04_endcheck =*+1
  lda #$00
  cmp $1035
  bne +
  lda #<sprites_05
  sta enable_sprites+1
  lda #>sprites_05
  sta enable_sprites+2
  clc
  lda $1035
  adc #$01
  sta .sp05_endcheck
+
  rts

sprites_05:
  lda snare_hit_col
  cmp #light_blue
  bne +
  lda #white
  !byte $2c
+
  lda #light_blue
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  eor #$08
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e

.sp05_endcheck =*+1
  lda #$00
  cmp $1035
  bne +
  lda #light_blue
  sta $d027
  sta $d028
  sta $d029
  sta $d02a
  eor #$08
  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e
  lda #<sprites_06
  sta enable_sprites+1
  lda #>sprites_06
  sta enable_sprites+2
+
  rts

!zone SPRITES_OUT
sprites_06:
.savex_00 =*+1
  ldx #$1f

  lda virtual_x0
  cmp #$a8
  beq ++
  inc virtual_x0
  lda sprite_anim_tab0,x
  sta vidmem0+$3f8
  sta vidmem1+$3f8
  sta vidmem0+$3f8+4
  sta vidmem1+$3f8+4
  dex
  bpl +
  ldx #$1f
+
  stx .savex_00
  jsr sprites_update

  rts
++
  lda #<sprites_07
  sta enable_sprites+1
  lda #>sprites_07
  sta enable_sprites+2

  rts
sprites_07:
.savex_01 =*+1
  ldx #$1f

  lda virtual_x1
  cmp #$a8
  beq ++
  inc virtual_x1
  lda sprite_anim_tab1,x
  sta vidmem0+$3f9
  sta vidmem1+$3f9
  sta vidmem0+$3f9+4
  sta vidmem1+$3f9+4
  dex
  bpl +
  ldx #$1f
+
  stx .savex_01
  jsr sprites_update

  rts
++
  lda #<sprites_08
  sta enable_sprites+1
  lda #>sprites_08
  sta enable_sprites+2

  rts
sprites_08:
.savex_02 =*+1
  ldx #$1f

  lda virtual_x2
  cmp #$a8
  beq ++
  inc virtual_x2
  lda sprite_anim_tab2,x
  sta vidmem0+$3fa
  sta vidmem1+$3fa
  sta vidmem0+$3fa+4
  sta vidmem1+$3fa+4
  dex
  bpl +
  ldx #$1f
+
  stx .savex_02
  jsr sprites_update

  rts
++
  lda #<sprites_09
  sta enable_sprites+1
  lda #>sprites_09
  sta enable_sprites+2

  rts

sprites_09:
.savex_03 =*+1
  ldx #$0f

  lda virtual_x3
  cmp #$a8
  beq ++
  inc virtual_x3
  lda sprite_anim_tab3,x
  sta vidmem0+$3fb
  sta vidmem1+$3fb
  sta vidmem0+$3fb+4
  sta vidmem1+$3fb+4
  dex
  bpl +
  ldx #$0f
+
  stx .savex_03
  jsr sprites_update
  rts
++
  lda #<sprites_10
  sta enable_sprites+1
  lda #>sprites_10
  sta enable_sprites+2
  rts

sprites_10:
.xdown =*+1
  ldx #$ff

  lda #%01010101
  sta charset+$000,x
  sta charset+$100,x
  sta charset+$200,x
  sta charset+$300,x
  sta charset+$400,x
  sta charset+$500,x
  sta charset+$600,x
  sta charset+$700,x

  dex
  cpx #$ff
  bne +

  lda #<fade_out
  sta enable_sprites+1
  lda #>fade_out
  sta enable_sprites+2

+
  stx .xdown
  rts
fade_out:
  lda #$01
  sta wait_end
  rts
; ______________________________________________________________________
!zone TABLES
  !src "tables.asm"





