!cpu 6510

!src "colorcodes.asm"
; ______________________________________________________________________
; ____________ ZEROPAGE ADRESSES

savea                 = $02
savex                 = $03
savey                 = $04
; ______________________________
;         $05 - $DF free to use!
; ______________________________
; $E0 - (max.) $F$  KRILL LOADER
; ______________________________
color                 = $f5

irq_ready             = $f6
d012_value            = $f7
d011_value            = $f8
irq_lo                = $f9
irq_hi                = $fa

global_ct             = $fb             ; $fb, $fc, $fd

; music player uses:    $fe, $ff

; ____________ RAM ADRESSES
code_start            = $0400
irq_addr              = $0f00
music_init            = $1000
music_play            = music_init+3
part_start_lo         = $2ffe
part_start_hi         = $2fff

; ____________ KRILL
  !src "loadersymbols-c64.inc"
; ______________________________________________________________________
; ______________________________________________ INCLUDES
*= music_init
  !bin "bcc2016.sid",,$7e
music_end:

*= loadraw
  !bin "loader-c64.c64",,2

*= $3000
  !bin "install-c64.c64",,2
krill_end:
; ______________________________________________________________________
; ______________________________________________ CODE START
  jmp basic_fade
reloc_code_01_start:
!pseudopc code_start {

  lda #$36
  sta $01

  jsr install

  lda #$35
  sta $01

  lda #<nmi
  sta $fffa
  lda #>nmi
  sta $fffb

  jsr init_vic

  lda #$00
  sta color
  sta global_ct
  sta global_ct+1
  sta global_ct+2
  sta d012_value
  sta d011_value

  lda #$00
  jsr music_init

  jsr init_timer

  lda #<irq
  sta $fffe
  sta irq_lo
  lda #>irq
  sta $ffff
  sta irq_hi

  asl $d019
  cli
; ________________________________________________ MAIN LOOP
!zone MAINLOOP
mainloop:
  lda #0
  sta loadaddrlo
  sta loadaddrhi
  ldx #<filename
  ldy #>filename

  jsr loadcompd

  lda $103c
-
  cmp $103c
  beq -

  inc filename+1

  lda part_start_lo
  sta exit+1
  lda part_start_hi
  sta exit+2

exit:
  jmp $0000

back2framework:
  jsr framework_wait_irq
  jsr init_vic

  jmp mainloop
; ____________
filename:
!scr "00"
!byte $00
; ____________
framework_wait_irq:
  lda #$00
  sta irq_ready
-
  lda irq_ready
  beq -
  rts
; ______________________________________________ INIT VIC
!zone INIT_VIC
init_vic:
  lda #$00
  tax
-
  sta $d000,x
  inx
  cpx #$2f
  bne -

  lda #$01
  sta $d01a
  rts
; ______________________________________________ INIT TIMER
!zone INIT_TIMER
init_timer:
  cmp $d012
  bne init_timer
  ldy #$08
  sty $dc04
-
	dey
  bne -
  sty $dc05
  sta $dc0e,y
  lda #$11
  cmp $d012
  sty $d015
  bne init_timer
  rts
}
reloc_code_01_end:
; ______________________________________________ IRQ
!zone IRQ
reloc_code_02_start:
!pseudopc irq_addr {
irq:
  sta savea
  stx savex
  sty savey

  lda color
  sta $d020
  sta $d021

  jsr music_play

  clc
  lda global_ct
  adc #$01
  sta global_ct
  lda global_ct+1
  adc #$00
  sta global_ct+1
  lda global_ct+2
  adc #$00
  sta global_ct+2

  lda d012_value
  sta $d012

  lda d011_value
  sta $d011

  lda irq_lo
  sta $fffe
  lda irq_hi
  sta $ffff

  lda #$01
  sta irq_ready

  asl $d019
  lda savea
  ldx savex
  ldy savey
nmi:
  rti
}
reloc_code_02_end:
; ______________________________________________________________________
!zone BASIC_FADE
basic_fade:
  sei
  lda #$7f
  sta $dc0d
  sta $dd0d
  lda $dc0d
  lda $dd0d

  lda $d020
  sta .bordercolor

fill_screen:
  lda #160
.mod1 =*+1
  sta $0400
.mod2 =*+1
  sta $07e7
.bordercolor =*+1
  lda #0
.mod3 =*+1
  sta $d800
.mod4 =*+1
  sta $d800+$03e7

  lda #$ff
-
  cmp $d012
  bne -

  clc
  lda .mod1
  adc #1
  sta .mod1
  bcc +
  inc .mod1+1
+
  sec
  lda .mod2
  sbc #1
  sta .mod2
  bcs +
  dec .mod2+1
+
  lda .mod1
  sta .mod3
  lda .mod2
  sta .mod4

  lda .mod1+1
  eor #($04 XOR $d8)
  sta .mod3+1

  lda .mod2+1
  eor #($04 XOR $d8)
  sta .mod4+1

  lda .mod1+1
  cmp #$06
  bne fill_screen

  lda #$0b
  sta $d011

  ldy .bordercolor

fade_d020:
  bit $d011
  bpl *-3
  bit $d011
  bmi *-3

  lda #$00
  sta $d020
.mod5 =*+1
  lda #$01
-
  cmp $d012
  bne -

  ldx #11
-
  dex
  bpl -

  sty $d020

  inc .mod5

  lda .mod5
  cmp #$00
  bne fade_d020

fade_d020_2:
  bit $d011
  bmi *-3

  lda #$00
  sta $d020

  bit $d011
  bpl *-3

.mod6 =*+1
  lda #$00
-
  cmp $d012
  bne -

  sty $d020

  inc .mod6

  lda .mod6
  cmp #37
  bne fade_d020_2

  lda #$00
  sta $d020
; ______________________________________________________________________
!zone COPY_CODE
  ldx #(reloc_code_01_end-reloc_code_01_start)
-
  lda reloc_code_01_start,x
  sta code_start,x
  dex
  cpx #$ff
  bne -

  ldx #(reloc_code_02_end-reloc_code_02_start)
-
  lda reloc_code_02_start,x
  sta irq_addr,x
  dex
  cpx #$ff
  bne -

  jmp code_start
