;; -*- mode: acme -*-
!cpu 6510

!if RELEASE = 1 {
  !src "../framework/frameworksymbols.inc"
}

; -----------------------------------------------------------------------------
; ---------------------------------------------------------------------- Macros

!macro fade .dir, .effect, .source, .destination, .size {

	lda #<.effect
	sta trn
	lda #>.effect
	sta trn+1

	ldx #$20

.loop:
	lda #<.source
	sta src
	lda #>.source
	sta src+1

	lda #<.destination
	sta dst
	lda #>.destination
	sta dst+1

	ldy #$00
	lda (trn),y
  +incw trn
	pha

	lda (trn),y
	+incw trn

	sta msk

!if .dir = 0 {
	lda #$ff
	eor msk
	sta msk
}
	pla
	tay

.step:
!if .dir = 1 {
	lda (src),y
	and msk
	ora (dst),y
} else {
	lda (dst),y
	and msk
}
	sta (dst),y

	+addw src, 8
	+addw dst, 8

	lda dst+1
	cmp #>(.destination+.size)
	bne .step

	lda dst
	cmp #<(.destination+.size)
	bne .step

	dex
	bne .loop
}

; -----------------------------------------------------------------------------

!macro copy .start, .end, .addr {

	lda #<.start
	sta src
	lda #>.start
	sta src+1

	lda #<.end
	sta end
	lda #>.end
	sta end+1

	lda #<.addr
	sta dst
	lda #>.addr
	sta dst+1

	jsr copy
}

; -----------------------------------------------------------------------------

!macro incw .addr {
  inc .addr
  bne +
  inc .addr+1
+
}

; -----------------------------------------------------------------------------

!macro addw .addr, .value {
  clc
  lda .addr
  adc #.value
  sta .addr
  bcc +
  inc .addr+1
+
}

; -----------------------------------------------------------------------------

!macro subw .addr, .value {
  sec
  lda .addr
  sbc #.value
  sta .addr
  bcs +
  dec .addr+1
+
}

; ______________________________________________________________________
; ____________ ZEROPAGE ADRESSES

savea                 = $02
savex                 = $03
savey                 = $04

; ______________________________
;         $05 - $DF free to use!
; ______________________________
; $E0 - (max.) $F$  KRILL LOADER
; ______________________________

color                 = $f5

irq_ready             = $f6
d012_value            = $f7
d011_value            = $f8
irq_lo                = $f9
irq_hi                = $fa

global_ct             = $fb             ; $fb, $fc, $fd

vicbase               = $4000
screen                = vicbase
sprite_pointer        = screen + $3f8
sprites               = vicbase + $1000
bitmap                = vicbase + $2000
data                  = $8000

src                   = $05
dst                   = $07
end                   = $09
trn                   = $0b
msk                   = $0d

; ______________________________________________________________________
; ______________________________________________ INCLUDES

!if RELEASE = 0 {
init_vic  = $046d

*= $0801
  !bin "../framework/framework-noloader-nokey.c64",,2
}

*= $2ffe
!byte <init_sines
!byte >init_sines

*= screen

!fi 1*$28, $02
!fi 13*$28, $03
!fi 1*$28, $02

!fi $400-(<*), $20

*= sprites
!bin "mayday.bin"
!bin "bcc2016.bin"
!bin "partyon.bin"

*= bitmap
!bin "bitmap.bin"

; ______________________________________________________________________
; ______________________________________________ CODE START

*= $3000
init_sines:

  lda #%00000010
  sta $dd00

  ; screen ram at vicbase+0, bitmap at vicbase+$2000 = $6000
  lda $d018
  and #$0f
  ora #$08
  sta $d018

  ; multicolor on
  lda $d016
  ora #$10
  sta $d016

  lda #$ff
  sta $d015

	lda #$00
  sta $d01b

  lda #$01
  sta $d027
  sta $d028
  sta $d029
  sta $d02a

  sta $d02b
  sta $d02c
  sta $d02d
  sta $d02e

  ldx #$00
  lda #$06
-
  sta $d800,x
  sta $d900,x
  sta $da00,x
  sta $db00,x
  dex
  bne -

  +copy bitmap+$1f40, bitmap+$1f40+1000, screen
  +copy bitmap+$2328, bitmap+$2328+1000, $d800
  +copy bitmap, bitmap+$2000, $8000
	jsr clearBitmap

  ldx #$07
-  txa
	ora #$40
  sta sprite_pointer,x
  dex
  bpl -

  lda #$00
  sta color

  ; hires mode on
  lda #$3b
  sta d011_value
	sta $d011

  lda #$1C
  sta d012_value

  lda #$00
  sta irq_ready
-
  lda irq_ready
  beq -

  lda #<_irq
  sta irq_lo
  lda #>_irq
  sta irq_hi

  +fade 1, effect, $8000, $6000, $2000

limbo:
	jmp *

done:

  lda #$00
  sta irq_ready
--
  lda irq_ready
  beq --

	lda #$00
	sta irq_lo
  sta d012_value
	lda #$0f
	sta irq_hi

	+fade 0, effect, $8000, $6000, $2000

	lda #$0b
	sta $d011
	sta d011_value

	lda #$00
	ldx #$00

!if RELEASE = 1 {
	jmp back2framework
}
	jmp *
; ______________________________________________________________________
; __________________________________________________________________ IRQ

!align 255,0

_irq:

  sta savea
  stx savex
  sty savey

!for i, 0, 7 {

  ldx index+i
  lda sine,x
	adc #$50
	sta $d001+i*2
  inc index+i

  lda dir+i
  bne +

  ;left

  +addw xpos+i*2, 1 ; 2
  lda xpos+i*2+1
  beq ++
  lda xpos+i*2
  cmp #$f0
  bcc ++

	+addw sprite_pointer+i, 8
  inc dir+i
	inc pass
	jmp ++

+ ;right

  +subw xpos+i*2, 8
  lda xpos+i*2+1
  bne ++
  lda #$20
  cmp xpos+i*2
  bcc ++

  dec dir+i

++ ; set

  lda xpos+i*2+1
  cmp #$02
  bcs ++++

  lda xpos+i*2
  sta $d000+i*2
  lda xpos+i*2+1
  bne +++

  ; low
  lda #(1<<i)
  eor #$ff
  and $d010
  sta $d010
  jmp ++++

+++ ; high
  lda $d010
  ora #(1<<i)
  sta $d010

++++
}

.exit:

  lda #$00
  sta $fffe
  lda #$0f
  sta $ffff

  lda #$00
  sta $d012

	lda pass
	cmp #$01
	bne +

	ldx #$07
-	lda crossed,x
	sta index,x
	dex
	bpl -

+	lda pass
	cmp #$09
	bne +

	ldx #$07
-	lda mixed,x
	sta index,x
	dex
	bpl -

+	lda pass
	cmp #$12
	bne +

	lda #<done
	sta limbo+1
	lda #>done
	sta limbo+2

+
  asl $d019
  lda savea
  ldx savex
  ldy savey

  rti

copy:
	ldy #$00

loop:
	lda (src),y
	sta (dst),y

	+incw src
	+incw dst

	lda end+1
	cmp src+1
	bne loop

	lda end
	cmp src
	bne loop

	rts

clearBitmap:
	ldy #$00
	lda #$00
	sta dst
	lda #$60
	sta dst+1

loop2:
	lda #$00
	sta (dst),y

+incw dst

	lda #$80
	cmp dst+1
	bne loop2

	rts

pass:	 !byte $00

index:	 !byte $00, $00, $00, $00, $00, $00, $00, $00
crossed: !byte $00, $80, $00, $80, $00, $80, $00, $80
mixed:   !byte $00, $f0, $e0, $d0, $c0, $b0, $a0, $90

xpos:
!for i, 0, 7 {
  !word $0180+i*$d0 ; $68
}

dir:
!byte $01, $01, $01, $01, $01, $01, $01, $01

sine:
!bin "sine.bin"

effect:
!bin "effect.bin"



